<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2023 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

namespace Drupal\key_apigee\Plugin\KeyInput;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\key\KeyInterface;

/**
 * Base class for Apigee Edge key input plugins.
 *
 * @internal This class is not part of the module's public programming API.
 */
abstract class ApigeeEdgeKeyInputBase extends ApigeeKeyInputBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    $config = parent::defaultConfiguration();
    $config += [
      'username' => '',
      'password' => '',
    ];
    return $config;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form_object = $form_state->getFormObject();
    assert($form_object instanceof EntityForm);
    $key = $form_object->getEntity();
    assert($key instanceof KeyInterface);
    $definition = $key->getKeyType()->getPluginDefinition();

    $form['username'] = [
      '#type' => 'textfield',
      '#title' => $definition['multivalue']['fields']['username']['label'],
      '#description' => $definition['multivalue']['fields']['username']['description'] ?? '' ,
      '#required' => $definition['multivalue']['fields']['username']['required'] ?? FALSE,
      '#default_value' => $this->getConfiguration()['username'],
      '#attributes' => ['autocomplete' => 'off'],
    ];
    $form['password'] = [
      '#type' => 'password',
      '#title' => $definition['multivalue']['fields']['password']['label'],
      '#description' => ($definition['multivalue']['fields']['password']['description'] ?? '') . !empty($this->getConfiguration()['password']) ? $this->t('Leave this empty to keep the saved value.') : '',
      '#required' => empty($this->getConfiguration()['password']) ? $definition['multivalue']['fields']['password']['required'] : FALSE,
      '#attributes' => [
        'autocomplete' => 'off',
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function ensureDefaultValuesAreStoredByKeyProviderEvenIfTheyAreNotOnInput(array &$values, FormStateInterface $form_state): void {
    if (empty($values['password'])) {
      $values['password'] = $this->getConfiguration()['password'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    // Set the default value if it was not provided on the UI.
    // \Drupal\key\Plugin\KeyInputBase::processSubmittedKeyValue() is not called
    // before submitted values are calculated, but it is called when they
    // are validated. ¯\_(ツ)_/¯.
    // @todo Open an issue about this in Key's issue queue.
    if (empty($form_state->getValues()['password'])) {
      $form_state->setValue('password', $this->getConfiguration()['password']);
    }
    parent::submitConfigurationForm($form, $form_state);
  }

}
