<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2023 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

namespace Drupal\key_apigee\Plugin\KeyInput;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\key\KeyInterface;

/**
 * Apigee Edge OAuth key input.
 *
 * @KeyInput(
 *    id = "apigee_edge_oauth",
 *    label = @Translation("Apigee Edge OAuth")
 *  )
 *
 * @internal This class is not part of the module's public programming API.
 */
final class ApigeeEdgeOAuthKeyInput extends ApigeeEdgeKeyInputBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    $config = parent::defaultConfiguration();
    $config['client_id'] = 'edgecli';
    $config['client_secret'] = 'edgeclisecret';
    $config['authorization_server'] = 'https://login.apigee.com/oauth/token';
    return $config;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form_object = $form_state->getFormObject();
    assert($form_object instanceof EntityForm);
    $key = $form_object->getEntity();
    assert($key instanceof KeyInterface);
    $definition = $key->getKeyType()->getPluginDefinition();

    $form['authorization_server'] = [
      '#type' => 'textfield',
      '#title' => $definition['multivalue']['fields']['authorization_server']['label'],
      '#description' => $definition['multivalue']['fields']['authorization_server']['description'] ?? '' ,
      '#required' => $definition['multivalue']['fields']['authorization_server']['required'] ?? FALSE,
      '#default_value' => $this->getConfiguration()['authorization_server'],
      '#attributes' => ['autocomplete' => 'off'],
    ];
    $form['client_id'] = [
      '#type' => 'textfield',
      '#title' => $definition['multivalue']['fields']['client_id']['label'],
      '#description' => $definition['multivalue']['fields']['client_id']['description'] ?? '' ,
      '#required' => $definition['multivalue']['fields']['client_id']['required'] ?? FALSE,
      '#default_value' => $this->getConfiguration()['client_id'],
      '#attributes' => ['autocomplete' => 'off'],
    ];
    $form['client_secret'] = [
      '#type' => 'password',
      '#title' => $definition['multivalue']['fields']['client_secret']['label'],
      '#description' => $definition['multivalue']['fields']['client_secret']['description'] ?? '' ,
      '#required' => $this->getConfiguration()['client_secret'] ?? NULL ? FALSE : ($definition['multivalue']['fields']['client_secret']['required'] ?? FALSE),
      '#default_value' => $this->getConfiguration()['client_secret'],
      '#attributes' => ['autocomplete' => 'off'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function ensureDefaultValuesAreStoredByKeyProviderEvenIfTheyAreNotOnInput(array &$values, FormStateInterface $form_state): void {
    parent::ensureDefaultValuesAreStoredByKeyProviderEvenIfTheyAreNotOnInput($values, $form_state);
    if (empty($values['authorization_server'])) {
      $values['authorization_server'] = $this->getConfiguration()['authorization_server'];
    }
    if (empty($values['client_id'])) {
      $values['client_id'] = $this->getConfiguration()['client_id'];
    }
    if (empty($values['client_secret'])) {
      $values['client_secret'] = $this->getConfiguration()['client_secret'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    if (empty($form_state->getValues()['authorization_server'])) {
      $form_state->setValue('authorization_server', $this->getConfiguration()['authorization_server']);
    }
    if (empty($form_state->getValues()['client_id'])) {
      $form_state->setValue('client_id', $this->getConfiguration()['client_id']);
    }
    if (empty($form_state->getValues()['client_secret'])) {
      $form_state->setValue('client_secret', $this->getConfiguration()['client_secret']);
    }
    parent::submitConfigurationForm($form, $form_state);
  }

}
