<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2023 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

namespace Drupal\key_apigee\Plugin\KeyInput;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\key\KeyInterface;
use Drupal\key\Plugin\KeyInputBase;

/**
 * Base class for Apigee key input plugins.
 *
 * @internal This class is not part of the module's public programming API.
 */
abstract class ApigeeKeyInputBase extends KeyInputBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'endpoint' => 'https://api.enterprise.apigee.com/v1',
      'organization' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form_object = $form_state->getFormObject();
    assert($form_object instanceof EntityForm);
    $key = $form_object->getEntity();
    assert($key instanceof KeyInterface);
    $definition = $key->getKeyType()->getPluginDefinition();

    $form['endpoint'] = [
      '#type' => 'textfield',
      '#title' => $definition['multivalue']['fields']['endpoint']['label'],
      '#description' => $definition['multivalue']['fields']['endpoint']['description'] ?? '' ,
      '#required' => $definition['multivalue']['fields']['endpoint']['required'] ?? FALSE,
      '#default_value' => $this->getConfiguration()['endpoint'],
      '#attributes' => ['autocomplete' => 'off'],
    ];

    $form['organization'] = [
      '#type' => 'textfield',
      '#title' => $definition['multivalue']['fields']['organization']['label'],
      '#description' => $definition['multivalue']['fields']['organization']['description'] ?? '' ,
      '#required' => $definition['multivalue']['fields']['organization']['required'] ?? FALSE,
      '#default_value' => $this->getConfiguration()['organization'],
      '#attributes' => ['autocomplete' => 'off'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  final public function processSubmittedKeyValue(FormStateInterface $form_state): array {
    $values = $form_state->getValues();
    $this->ensureDefaultValuesAreStoredByKeyProviderEvenIfTheyAreNotOnInput($values, $form_state);

    return [
      'submitted' => Json::encode($values),
      'processed_submitted' => Json::encode($values),
    ];
  }

  /**
   * Ensure default values are stored by a key provider even.
   *
   * Even if they are not on input.
   *
   * @param array $values
   *   Plugin related values from form states that gets persisted to by a key
   *   provider that accepts and stores input.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state with incoming form values.
   */
  protected function ensureDefaultValuesAreStoredByKeyProviderEvenIfTheyAreNotOnInput(array &$values, FormStateInterface $form_state): void {}

}
