<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2023-2024 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

namespace Drupal\key_apigee\Plugin\KeyType;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\FormStateInterface;
use Drupal\key\KeyInterface;
use Drupal\key\Plugin\KeyType\AuthenticationMultivalueKeyType;
use Drupal\key_apigee\ConnectionTester\ConnectionFailedException;
use Drupal\key_apigee\ConnectionTester\ConnectionTesterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for Apigee authentication key types.
 *
 * @internal This class is not part of the module's public programming API.
 */
abstract class ApigeeAuthKeyTypeBase extends AuthenticationMultivalueKeyType implements ApigeeAuthKeyTypeInterface {
  /**
   * The connection tester.
   *
   * @var \Drupal\key_apigee\ConnectionTester\ConnectionTesterInterface
   *
   * @todo Mark this private after https://www.drupal.org/project/drupal/issues/3110266
   *   is merged and minimum required version is bumped to that Drupal core
   *   version.
   */
  protected ConnectionTesterInterface $connectionTester;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    $instance->connectionTester = $container->get('key_apigee.connection_tester');
    $instance->stringTranslation = $container->get('string_translation');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function validateKeyValue(array $form, FormStateInterface $form_state, $key_value): void {
    // We do not need to call parent validation to ensure required fields are
    // set because that does not handle od-demand form validation, but
    // the built-in form validation handlers do.
    if ($form_state->getErrors() !== []) {
      return;
    }

    try {
      $this->connectionTester->testWithKeyValue($this, (string) $key_value);
    }
    catch (ConnectionFailedException $e) {
      $form_state->setError($form['settings']['input_section']['key_input_settings'], $this->t('Apigee could not be reached with the provided credentials. See further details below. <pre>%reason</pre>', ['%reason' => $e->getMessage()]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function generateKeyValue(array $configuration): string {
    return Json::encode($configuration);
  }

  /**
   * {@inheritdoc}
   */
  public function getEndpoint(KeyInterface $key): string {
    $endpoint = $key->getKeyValues()['endpoint'] ?? NULL;
    assert($endpoint !== NULL, 'API endpoint is set on the Apigee key.');

    return $endpoint;
  }

  /**
   * {@inheritdoc}
   */
  public function getOrganization(KeyInterface $key): string {
    $organization = $key->getKeyValues()['organization'] ?? NULL;
    assert($organization !== NULL, 'Organization is set on the Apigee key.');

    return $organization;
  }

}
