<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2023 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

namespace Drupal\key_apigee\Plugin\KeyType;

use Drupal\Core\Form\FormStateInterface;
use Drupal\key\KeyInterface;

/**
 * Apigee X/Hybrid authentication key type.
 *
 * @\Drupal\key_apigee\Annotation\ApigeeXAuth(
 *   id = \Drupal\key_apigee\Plugin\KeyType\ApigeeXAuthKeyType::KEY_TYPE_ID,
 *   label = @Translation("Apigee X/Hybrid")
 * )
 *
 * @internal This class is not part of the module's public programming API.
 */
final class ApigeeXAuthKeyType extends ApigeeAuthKeyTypeBase {

  public const KEY_TYPE_ID = 'apigee_x_auth';

  /**
   * {@inheritdoc}
   */
  public function validateKeyValue(array $form, FormStateInterface $form_state, $key_value): void {
    if ($form_state->getErrors() !== []) {
      return;
    }

    $data = $this->unserialize($key_value);
    $is_valid = json_validate($data['service_account_key']);
    if ($is_valid) {
      $data = json_decode($data['service_account_key'], TRUE);
      if (!array_key_exists('client_email', $data) || empty($data['client_email'])) {
        $form_state->setError($form['settings']['input_section']['key_input_settings']['service_account_key'], $this->t('The provided value is invalid. The "client_email" is missing.'));
        $is_valid = FALSE;
      }

      if (!array_key_exists('private_key', $data) || empty($data['private_key'])) {
        $form_state->setError($form['settings']['input_section']['key_input_settings']['service_account_key'], $this->t('The provided value is invalid. The "private_key" is missing.'));
        $is_valid = FALSE;
      }
    }
    else {
      $form_state->setError($form['settings']['input_section']['key_input_settings']['service_account_key'], $this->t('The provided value is not a valid JSON. See further details below. <pre>%reason</pre>', ['%reason' => json_last_error_msg()]));
    }

    if ($is_valid) {
      parent::validateKeyValue($form, $form_state, $key_value);
    }

  }

  /**
   * Gets the GCP service account key from an Apigee key entity.
   *
   * @param \Drupal\key\KeyInterface $key
   *   An Apigee key entity.
   *
   * @return string
   *   The GCP service account key in JSON format.
   */
  public function getServiceAccountKey(KeyInterface $key): string {
    $service_account_key = $key->getKeyValues()['service_account_key'] ?? NULL;
    assert($service_account_key !== NULL, 'GCP service account key is set on the Apigee X/Hybrid key.');

    return $service_account_key;
  }

}
