<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2024 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */
namespace Drupal\Tests\key_apigee\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\key_apigee\Traits\ApigeeKeyDataProviderTrait;
use Drupal\key_apigee\Plugin\KeyType\ApigeeEdgeBasicAuthKeyType;
use Drupal\key_apigee_test\FakeConnectionTester;
use Drupal\user\UserInterface;

/**
 * Test Apigee Key related interactions on the UI.
 *
 * @internal This class is not part of the module's public programming API.
 */
final class ApigeeKeyUITest extends WebDriverTestBase {

  use ApigeeKeyDataProviderTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['key', 'key_apigee'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with the 'administer keys' permission.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $keyAdminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->keyAdminUser = $this->drupalCreateUser(['administer keys']);
  }

  /**
   * Test that create/edit forms works expectedly.
   *
   * This must be tested via real browser interactions because
   * \Drupal\Tests\UiHelperTrait::submitForm() would hide underlying issues in
   * Key's architectural design.
   *
   * @dataProvider minimalApigeeKeyInputDataProvider
   */
  public function testCreateEditApigeeKey(string $key_type_id, array $form_input, array $expected_config_stored): void {
    $this->drupalLogin($this->keyAdminUser);
    $this->drupalGet('admin/config/system/keys/add');

    $page = $this->getSession()->getPage();
    $page->findField('Key name')->setValue($key_type_id);

    $page->findField('Key type')->setValue($key_type_id);
    $this->assertSession()->assertWaitOnAjaxRequest();
    foreach ($form_input as $key => $value) {
      $page->findField('key_input_settings[' . $key . ']')->setValue($value);
    }

    $page->pressButton('Save');
    $this->assertSession()->pageTextContains(sprintf('The key %s has been added.', $key_type_id));
    self::assertEquals($expected_config_stored, json_decode($this->config("key.key.{$key_type_id}")->get('key_provider_settings')['key_value'], TRUE, flags: JSON_THROW_ON_ERROR), 'The key got created with the expected content.');

    $this->drupalGet("admin/config/system/keys/manage/{$key_type_id}");
    $page->pressButton('Edit');
    $page->pressButton('Save');
    // This covers workarounds in
    // \Drupal\key_apigee\Plugin\KeyInput\ApigeeKeyInputBase::ensureDefaultValuesAreStoredByKeyProviderEvenIfTheyAreNotOnInput().
    self::assertEquals($expected_config_stored, json_decode($this->config("key.key.{$key_type_id}")->get('key_provider_settings')['key_value'], TRUE, flags: JSON_THROW_ON_ERROR), 'There is no data loss when edit form is saved without modifications.');
  }

  /**
   * Tests that connection errors bubble up as form validation errors.
   */
  public function testConnectionErrorsBubbleUpAsFormValidationErrors(): void {
    \Drupal::service('module_installer')->install(['key_apigee_test']);
    $connection_tester = \Drupal::service('key_apigee.connection_tester');
    self::assertInstanceOf(FakeConnectionTester::class, $connection_tester);
    $this->drupalLogin($this->keyAdminUser);
    $this->drupalGet('admin/config/system/keys/add');

    $page = $this->getSession()->getPage();
    $page->findField('Key name')->setValue(ApigeeEdgeBasicAuthKeyType::KEY_TYPE_ID);

    $page->findField('Key type')->setValue(ApigeeEdgeBasicAuthKeyType::KEY_TYPE_ID);
    $this->assertSession()->assertWaitOnAjaxRequest();
    $form_input = [
      'endpoint' => 'https://api.enterprise.apigee.com/v1',
      'organization' => 'acme',
      'username' => 'superadmin',
      'password' => '12345',
    ];
    foreach ($form_input as $key => $value) {
      $page->findField('key_input_settings[' . $key . ']')->setValue($value);
    }

    $connection_tester->startFakingConnectionErrors();
    $page->pressButton('Save');

    $this->assertSession()->pageTextNotContains(sprintf('The key %s has been added.', ApigeeEdgeBasicAuthKeyType::KEY_TYPE_ID));
    $this->assertSession()->pageTextContains('Apigee could not be reached with the provided credentials. See further details below. Faked connection error');

    $connection_tester->stopFakingConnectionErrors();
    $page->findField('key_input_settings[password]')->setValue('12345');
    $page->pressButton('Save');
    $this->assertSession()->pageTextContains(sprintf('The key %s has been added.', ApigeeEdgeBasicAuthKeyType::KEY_TYPE_ID));
  }

}
