<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2024 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */
namespace Drupal\Tests\key_apigee\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\key_apigee\Traits\ApigeeKeyDataProviderTrait;
use Drupal\key\Entity\Key;
use Drupal\key_apigee\Repository\ApigeeKeyRepository;

/**
 * Tests methods on the Apigee key repository.
 *
 * @covers \Drupal\key_apigee\Repository\ApigeeKeyRepository
 *
 * @internal This class is not part of the module's public programming API.
 */
final class ApigeeKeyRepositoryTest extends KernelTestBase {

  use ApigeeKeyDataProviderTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'key',
    'key_apigee',
  ];

  /**
   * The Apigee key repository.
   *
   * @var \Drupal\key_apigee\Repository\ApigeeKeyRepository
   */
  private ApigeeKeyRepository $apigeeKeyRepository;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->apigeeKeyRepository = $this->container->get('key_apigee.repository');
  }

  /**
   * Test fetchAll() returns all keys that are Apigee keys (by key type).
   *
   * @dataProvider fullApigeeKeyInputDataProvider
   */
  public function testFetchAllReturnsAllApigeeKeys(string $key_type_id, array $settings): void {
    $key = Key::create([
      'id' => $key_type_id,
      'label' => $key_type_id,
      'key_type' => $key_type_id,
      'key_type_settings' => [],
      'key_provider' => 'config',
      'key_provider_settings' => ['key_value' => json_encode($settings, JSON_THROW_ON_ERROR)],
      'key_input' => $key_type_id,
    ]);
    $key->save();

    $items = $this->apigeeKeyRepository->fetchAll();
    $item = reset($items);
    self::assertEquals($key->id(), $item->id());
  }

  /**
   * Test fetchAll() does not return a non-Apigee key (by key type).
   */
  public function testFetchAllDoesNotReturnNonApigeeKey(): void {
    $key = Key::create([
      'id' => 'authentication',
      'label' => 'authentication',
      'key_type' => 'authentication',
      'key_type_settings' => [],
      'key_provider' => 'config',
      'key_provider_settings' => ['key_value' => ''],
      'key_input' => 'text_field',
    ]);
    self::assertEquals(1, $key->save());

    self::assertEmpty($this->apigeeKeyRepository->fetchAll());
  }

  /**
   * Test getById() returns all kinds of valid Apigee keys.
   *
   * @dataProvider fullApigeeKeyInputDataProvider
   */
  public function testGetByIdReturnsAllKindsOfValidApigeeKeys(string $key_type_id, array $settings): void {
    $key = Key::create([
      'id' => $key_type_id,
      'label' => $key_type_id,
      'key_type' => $key_type_id,
      'key_type_settings' => [],
      'key_provider' => 'config',
      'key_provider_settings' => ['key_value' => json_encode($settings, JSON_THROW_ON_ERROR)],
    ]);
    $key->save();

    self::assertEquals($key->id(), $this->apigeeKeyRepository->getById($key->id())->id());
  }

  /**
   * Test getById() throws an exception for a non Apigee key.
   */
  public function testGetByIdThrowsAnExceptionForNonApigeeKey(): void {
    $this->expectException(\LogicException::class);
    $key = Key::create([
      'id' => 'authentication',
      'label' => 'authentication',
      'key_type' => 'authentication',
      'key_type_settings' => [],
      'key_provider' => 'config',
      'key_provider_settings' => ['key_value' => ''],
      'key_input' => 'text_field',
    ]);
    self::assertEquals(1, $key->save());

    self::assertEmpty($this->apigeeKeyRepository->getById($key->id()));
  }

}
