<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2024 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

namespace Drupal\Tests\key_apigee\Traits;

use Drupal\key_apigee\Plugin\KeyType\ApigeeEdgeBasicAuthKeyType;
use Drupal\key_apigee\Plugin\KeyType\ApigeeEdgeOAuthKeyType;
use Drupal\key_apigee\Plugin\KeyType\ApigeeXAuthKeyType;
use Drupal\key_apigee\Plugin\KeyType\ApigeeXOnGceHostedSiteAuthKeyType;

/**
 * Trait that provides Apigee Key data for testing.
 *
 * @internal This trait is not part of the module's public programming API.
 */
trait ApigeeKeyDataProviderTrait {

  /**
   * Provides minimal input data containing only required fields.
   *
   * Fields with default values are excluded!
   *
   * @return \Generator<string, array>
   *   The array contains:
   *   * the Apigee key type plugin id.
   *   * the minimal input data for the given Apigee key type containing only
   *     required fields and fields without values are excluded,
   *   * the expected stored state after key is created.
   */
  public function minimalApigeeKeyInputDataProvider(): \Generator {
    $edge_common_settings = [
      'endpoint' => 'https://api.enterprise.apigee.com/v1',
      'organization' => 'acme',
    ];
    $edge_basic_auth_settings = $edge_common_settings + [
      'username' => 'superadmin',
      'password' => '12345',
    ];

    yield ApigeeEdgeBasicAuthKeyType::KEY_TYPE_ID => [
      ApigeeEdgeBasicAuthKeyType::KEY_TYPE_ID,
      $edge_basic_auth_settings,
      $edge_basic_auth_settings,
    ];

    yield ApigeeEdgeOAuthKeyType::KEY_TYPE_ID => [
      ApigeeEdgeOAuthKeyType::KEY_TYPE_ID,
      $edge_basic_auth_settings,
      $edge_basic_auth_settings + [
        'client_id' => 'edgecli',
        'client_secret' => 'edgeclisecret',
        'authorization_server' => 'https://login.apigee.com/oauth/token',
      ],
    ];

    yield ApigeeXOnGceHostedSiteAuthKeyType::KEY_TYPE_ID => [
      ApigeeXOnGceHostedSiteAuthKeyType::KEY_TYPE_ID,
      [
        'organization' => 'acme',
      ],
      [
        'organization' => 'acme',
        'endpoint' => 'https://apigee.googleapis.com/v1',
      ],
    ];

    yield ApigeeXAuthKeyType::KEY_TYPE_ID => [
      ApigeeXAuthKeyType::KEY_TYPE_ID,
      [
        'organization' => 'acme',
        'service_account_key' => json_encode(['client_email' => 'foobar@example.com', 'private_key' => 'secret'], JSON_THROW_ON_ERROR),
      ],
      [
        'organization' => 'acme',
        'service_account_key' => json_encode(['client_email' => 'foobar@example.com', 'private_key' => 'secret'], JSON_THROW_ON_ERROR),
        'endpoint' => 'https://apigee.googleapis.com/v1',
      ],
    ];
  }

  /**
   * Provides Apigee key data for all fields, even to defaults.
   *
   * @return \Generator<string, array>
   *   The key type plugin id and a valid payload for Key::create().
   */
  public function fullApigeeKeyInputDataProvider(): \Generator {
    $edge_common_settings = [
      'endpoint' => 'https://api.enterprise.apigee.com/v1',
      'organization' => 'acme',
    ];
    $edge_basic_auth_settings = $edge_common_settings + [
      'username' => 'superadmin',
      'password' => '12345',
    ];

    yield ApigeeEdgeBasicAuthKeyType::KEY_TYPE_ID => [
      ApigeeEdgeBasicAuthKeyType::KEY_TYPE_ID,
      $edge_basic_auth_settings,
    ];

    yield ApigeeEdgeOAuthKeyType::KEY_TYPE_ID => [
      ApigeeEdgeOAuthKeyType::KEY_TYPE_ID,
      $edge_basic_auth_settings + [
        'client_id' => 'user',
        'client_secret' => 'secret',
        'authorization_server' => 'https://127.0.0.1:8080',
      ],
    ];

    yield ApigeeXOnGceHostedSiteAuthKeyType::KEY_TYPE_ID => [
      ApigeeXOnGceHostedSiteAuthKeyType::KEY_TYPE_ID,
      [
        'organization' => 'acme',
        'endpoint' => 'https://127.0.0.1:8080',
      ],
    ];

    yield ApigeeXAuthKeyType::KEY_TYPE_ID => [
      ApigeeXAuthKeyType::KEY_TYPE_ID,
      [
        'organization' => 'acme',
        'endpoint' => 'https://127.0.0.1:8080',
        'service_account_key' => json_encode(['client_email' => 'foobar@example.com', 'private_key' => 'secret'], JSON_THROW_ON_ERROR),
      ],
    ];
  }

}
